;*** transmit.inc ***;
;------------------------------------------------
; Author: Torsten Knorr
; E-mail: create-soft@freenet.de
; copyright(C) 2016-2017 by Torsten Knorr
;------------------------------------------------

 .ifndef __RFM12BP_TRANSMIT__
 .equ    __RFM12BP_TRANSMIT__ = 1

 .include "RFM12BP.inc"

;------------------------------------------------
; RFM12BP PIN 8 SDO -> ATmega48 PD2 INT0

 .macro M_INIT_INT0_TRANSMIT

; Interrupt 0 sense control
; ISC01 ISC00 Description
;     0     0 The low level of INT0 generates an interrupt request
;     0     1 Any logical change on INT0 generates an interrupt request
;     1     0 The falling edge of INT0 generates an interrupt request
;     1     1 The rising edge of INT0 generates an interrupt request
    lds  R_TEMP_L,      EICRA
    sbr  R_TEMP_L,      ((1 << ISC00) | (1 << ISC01))
    sts  EICRA,         R_TEMP_L

; disable INT0
    cbi  EIMSK,         INT0_EN_TX

 .endmacro

;------------------------------------------------

 .macro M_RFM12BP_INT0_TRANSMIT

    in   RI_SREG,       SREG

; check pointer     >=
    cp   RI_XP_RTX_L,   RI_XP_END_L
    cpc  RI_XP_RTX_H,   RI_XP_END_H
    brsh RFM12BP_INT0_TRANSMIT_RETURN
    ld   RI_TX_BYTE,    X+

    out  SPDR,          RI_TX_BYTE
    M_SPI_WAIT_READY
    in   RI_RX_BYTE,    SPDR

 RFM12BP_INT0_TRANSMIT_RETURN:
    out  SREG,          RI_SREG
    reti

 .endmacro

;------------------------------------------------

 .macro M_RFM12BP_TRANSMIT

    TWI_DISABLE_ACK

    M_SPI_MT_CMD (CMD_SET_POWER | EN_TX | EN_SYNTH | EN_CRYSTAL | DI_CLK_PIN)
    sbi  PORTC,         RFM12BP_EN_TX
    M_SPI_TX_CONFIG_1

    ldi  R_PARA_L,      low(@0)
    ldi  R_PARA_H,      high(@0)
    ldi  RI_XP_RTX_L,   low(@1)
    ldi  RI_XP_RTX_H,   high(@1)

    movw RI_XP_END_L,   RI_XP_RTX_L
    add  RI_XP_END_L,   R_PARA_L
    adc  RI_XP_END_H,   R_PARA_H

    ldi  R_COUNT_L,     BYTES_PRE_SIG

    cbi  SPI_PORT,      SPI_SS
    M_SPI_MT_WORD (CMD_TRANSMIT | 0x0F)
    dec  R_COUNT_L

 RFM12BP_WAIT_PRE_SIGNAL_START:
    sbis PINB,          SPI_MISO
    rjmp RFM12BP_WAIT_PRE_SIGNAL_START

 RFM12BP_PRE_SIGNAL_LOOP:

    M_SPI_MT_BYTE 0x0F
 RFM12BP_WAIT_PRE_SIGNAL:
    sbis PINB,          SPI_MISO
    rjmp RFM12BP_WAIT_PRE_SIGNAL

    dec  R_COUNT_L
    brne RFM12BP_PRE_SIGNAL_LOOP
    sbi  SPI_PORT,     SPI_SS

    M_SPI_TX_CONFIG_2

    cbi  SPI_PORT,      SPI_SS
    M_SPI_MT_WORD (CMD_TRANSMIT | 0xAA)

; reset interrupt flag
    sbi  EIFR,          INTF0
; enable INT0 transmit
    sbi  EIMSK,         INT0_EN_TX

   RFM12BP_WAIT_TRANSMIT:
    cp   RI_XP_RTX_L,   RI_XP_END_L
    cpc  RI_XP_RTX_H,   RI_XP_END_H
    brlo RFM12BP_WAIT_TRANSMIT

; disable INT0 transmit
    cbi  EIMSK,         INT0_EN_TX

 RFM12BP_WAIT_LAST_BYTE:
    sbis PINB,          SPI_MISO
    rjmp RFM12BP_WAIT_LAST_BYTE

    sbi  SPI_PORT,      SPI_SS

    M_SPI_MT_CMD (CMD_SET_POWER | EN_SYNTH | EN_CRYSTAL | DI_CLK_PIN)

    cbi  PORTC,         RFM12BP_EN_TX
    TWI_ENABLE_ACK

 .endmacro

;------------------------------------------------

 .endif

;------------------------------------------------
