/*** DataRequest.cc ***/
//------------------------------------------------
// Author: Torsten Knorr
// E-mail: create-soft@freenet.de
// Copyright (c) 2015-2017, Torsten Knorr
//------------------------------------------------

 #include "Lib//WeatherStation.h"
 #include "Lib//LCD_Command.h"
 #include "Lib//I2C.cc"
 #include "Lib//DataRequestThreadDisplay.cc"
 #include "Lib//DataRequestThreadKeyboard.cc"
 #include "Lib//InsertValues.cc"
 #include "Lib//SetParameter.cc"
 #include "Lib//Time.cc"

//------------------------------------------------

 void MemCopy $asm("TagMemCopy") (float p_f_array[], byte p_by_array[], word w_bytes_count);

//------------------------------------------------

 byte gby_id_byte_4;
 byte gby_cmd;
 byte gby_para_1;
 byte gby_para_2;
 byte gby_display_run;
 byte gaby_data_buff[RFM12BP_BUFF_SIZE];
 float gaf_values[RFM12BP_VALUES_COUNT];

//------------------------------------------------

 BOOL DataRequest(void)
    {
    int i_index;
    byte aby_time[7];
    byte by_crc;
    byte aby_request[RFM12BP_REQUEST_SIZE];
    byte by_ret_value;

    by_ret_value = true;

    for(i_index = 0; i_index < RFM12BP_BUFF_SIZE; i_index++)
        gaby_data_buff[i_index] = 0;

    aby_request[0]  = ID_BYTE_1;
    aby_request[1]  = ID_BYTE_2;
    aby_request[2]  = ID_BYTE_3;
    aby_request[3]  = gby_id_byte_4;

    aby_request[4]  = Clock_GetVal(CLOCK_SEC);
    aby_request[5]  = Clock_GetVal(CLOCK_MIN);
    aby_request[6]  = Clock_GetVal(CLOCK_HOUR);
    aby_request[7]  = Clock_GetVal(CLOCK_DAY);
    aby_request[8]  = Clock_GetVal(CLOCK_MON);
    aby_request[9]  = Clock_GetVal(CLOCK_YEAR);

    aby_request[10] = gby_cmd;
    aby_request[11] = gby_para_1;
    aby_request[12] = gby_para_2;

    if(!I2CTransmitData(RFM12BP_ADDR, aby_request, RFM12BP_REQUEST_SIZE, true))
        {
        SET_DISPLAY_MSG("ERR I2C ", "Tran Req", 200)
        by_ret_value = false;
        }

    SET_DISPLAY_MSG("  Wait  ", "Get Data", 100)

    if(!I2CReceiveData((RFM12BP_ADDR + 1), gaby_data_buff, RFM12BP_BUFF_SIZE, false))
        {
        SET_DISPLAY_MSG("ERR I2C ", "Rece Res", 200)
        by_ret_value = false;
        }


    for(i_index = 0; i_index < RFM12BP_REQUEST_SIZE; i_index++)
        {
        Msg_WriteHex(aby_request[i_index]);
        Msg_WriteChar(' ');
        }
    Msg_WriteChar(13);

    for(i_index = 0; i_index < RFM12BP_REQUEST_SIZE; i_index++)
        {
        Msg_WriteHex(gaby_data_buff[i_index]);
        Msg_WriteChar(' ');
        }
    Msg_WriteChar(13);
    Msg_WriteChar(13);


    for(i_index = 0; i_index < RFM12BP_REQUEST_SIZE; i_index++)
        if(gaby_data_buff[i_index] != aby_request[i_index])
            {
            SET_DISPLAY_MSG("ERR Data", " Header ", 100)
            Thread_Delay(100);
            by_ret_value = false;
            break;
            }

    //Msg_WriteChar(' ');
    by_crc = 0;
    for(; i_index < (RFM12BP_BUFF_SIZE - 1); i_index++)
        {
        by_crc = by_crc ^ gaby_data_buff[i_index];
        //Msg_WriteHex(gaby_data_buff[i_index]);
        //if(!(i_index % 8))
        //    Msg_WriteChar(13);
        //Msg_WriteChar(' ');
        }

/*
    Msg_WriteHex(by_crc);
    Msg_WriteText(" : ");
    Msg_WriteHex(gaby_data_buff[i_index]);
    Msg_WriteChar(13);
*/

    if(by_crc != gaby_data_buff[i_index])
        {
        SET_DISPLAY_MSG("ERR Data", "Checksum", 100)
        Thread_Delay(100);
        by_ret_value = false;
        }

    MemCopy(gaf_values, (gaby_data_buff + RFM12BP_REQUEST_SIZE), RFM12BP_DATA_SIZE);

    Thread_Delay(100);

    return by_ret_value;
    }

//------------------------------------------------

 void main(void)
    {

// I2C SCL+SCA Ports pull up on
    Port_DataDirBit(24, 0);
    Port_DataDirBit(25, 0);
    Port_WriteBit(24, 1);
    Port_WriteBit(25, 1);

    gby_wait_display = false;
    gi_display_mode  = DISPLAY_CLOCK;
    gi_menu          = MENU_CLOCK;

    gby_id_byte_4 = 0x01;
    gby_cmd       = 0x00;
    gby_para_1    = 0x00;
    gby_para_2    = 0x00;

    Port_DataDirBit(PORT_LED1, PORT_OUT);
    Port_DataDirBit(PORT_LED2, PORT_OUT);
    Port_WriteBit(PORT_LED1, LED_ON);
    Thread_Delay(20);
    Port_WriteBit(PORT_LED2, LED_ON);
    Thread_Delay(20);
    Port_WriteBit(PORT_LED1, LED_OFF);
    Thread_Delay(20);
    Port_WriteBit(PORT_LED2, LED_OFF);

    LCD_Init();
    Thread_Delay(LCD_DELAY);

    Thread_Start(THREAD_KEYBOARD, ThreadKeyboard);
    Thread_Start(THREAD_DISPLAY, ThreadDisplay);

    I2C_Init(I2C_100kHz);
    SetSystemTime();
    while(1)
        {
        gby_display_run = gby_display_run ^ 0x01;
        Thread_Delay(50);
        }

    return;
    }
//------------------------------------------------