/*** DataRequestThreadDisplay.cc ***/
//------------------------------------------------
// Author: Torsten Knorr
// E-mail: create-soft@freenet.de
// Copyright (c) 2015-2017, Torsten Knorr
//------------------------------------------------

 #include "WeatherStation.h"

//------------------------------------------------

 int   gi_display_mode;
 byte  gby_wait_display;
 char  gac_line_1[41];
 char  gac_line_2[41];
 byte  gby_cursor_pos;
 word  gw_thread_delay;
 float gf_value;

//------------------------------------------------

 void DisplayClear(void)
    {

    LCD_ClearLCD();
    Thread_Delay(LCD_DELAY);

    LCD_CursorPos(0x00);
    Thread_Delay(LCD_DELAY);

    LCD_CursorOff();
    Thread_Delay(LCD_DELAY);

    return;
    }

//------------------------------------------------

 void DisplayInsert(void)
    {

    DisplayClear();

    LCD_WriteText(gac_line_1);
    Thread_Delay(LCD_DELAY);

    LCD_CursorPos(0x40);
    Thread_Delay(LCD_DELAY);

    LCD_WriteText(gac_line_2);
    Thread_Delay(200);

    LCD_CursorPos(0x40);
    Thread_Delay(LCD_DELAY);

    LCD_WriteText(" :      ");
    Thread_Delay(LCD_DELAY);

    LCD_CursorPos(0x42);
    Thread_Delay(LCD_DELAY);

    LCD_CursorOn();
    Thread_Delay(LCD_DELAY);

    gby_wait_display = false;
    gi_display_mode = DISPLAY_STOP;

    return;
    }

//------------------------------------------------

 void DisplayLine1(void)
    {

    LCD_WriteRegister(LCD_RETURN, 0x00);
    Thread_Delay(LCD_DELAY);
    LCD_WriteText(gac_line_1);
    Thread_Delay(gw_thread_delay);

    return;
    }

//------------------------------------------------

 void DisplayLine2(void)
    {

    LCD_WriteRegister(LCD_RETURN, 0x00);
    Thread_Delay(LCD_DELAY);

    LCD_CursorPos(0x40);
    Thread_Delay(LCD_DELAY);

    LCD_WriteText(gac_line_2);
    Thread_Delay(gw_thread_delay);

    return;
    }

//------------------------------------------------

 void DisplayMsg(void)
    {

    DisplayClear();

    LCD_WriteText(gac_line_1);
    Thread_Delay(LCD_DELAY);

    LCD_CursorPos(0x40);
    Thread_Delay(LCD_DELAY);

    LCD_WriteText(gac_line_2);
    Thread_Delay(gw_thread_delay);

    gi_display_mode = DISPLAY_STOP;

    return;
    }

//------------------------------------------------

 void DisplayMoveText(void)
    {
    byte by_count;
    byte by_count_shifts;
    byte by_str_length_1;
    byte by_str_length_2;

    by_str_length_1 = Str_Len(gac_line_1);
    by_str_length_2 = Str_Len(gac_line_2);

    if(by_str_length_1 > by_str_length_2)
        by_count_shifts = by_str_length_1 - 8;
    else
        by_count_shifts = by_str_length_2 - 8;

    DisplayClear();

    LCD_WriteText(gac_line_1);
    Thread_Delay(LCD_DELAY);

    LCD_CursorPos(0x40);
    Thread_Delay(LCD_DELAY);

    LCD_WriteText(gac_line_2);
    Thread_Delay(100);

    for(by_count = 0; by_count < by_count_shifts; by_count++)
        {
        LCD_WriteRegister((LCD_SHIFT | SHIFT_DISPLAY | SHIFT_LEFT), 0x00);
        Thread_Delay(25);
        }
    Thread_Delay(100);

    return;
    }

//------------------------------------------------

 void DisplayValue(void)
    {
    char ac_value[10];

    DisplayClear();

    LCD_WriteText(gac_line_1);
    Thread_Delay(LCD_DELAY);

    LCD_CursorPos(0x40);
    Thread_Delay(LCD_DELAY);

    if((MIN_VALUE < gf_value) && (gf_value < MAX_VALUE))
        Str_Printf(ac_value, "%8.2f", gf_value);
    else
        ac_value = "INVALID!";

    LCD_WriteText(ac_value);
    Thread_Delay(LCD_DELAY);

    gi_display_mode = DISPLAY_STOP;

    return;
    }

//------------------------------------------------

 BOOL DisplayResult(BOOL bo_success, char p_string[])
    {

    DisplayClear();

    LCD_WriteText(p_string);
    Thread_Delay(LCD_DELAY);

    LCD_CursorPos(0x40);
    Thread_Delay(LCD_DELAY);

    if(bo_success)
        {
        LCD_WriteText("-- OK --");
        Thread_Delay(LCD_DELAY);
        }
    else
        {
        LCD_WriteText( " ERROR  ");
        Thread_Delay(LCD_DELAY);
        }

    Thread_Delay(200);

    return bo_success;
    }

//------------------------------------------------

 void DisplaySystemTime(void)
    {
    char ac_time[9];
    byte by_clock;

    by_clock = Clock_GetVal(CLOCK_HOUR);
    ac_time[0] = HI_CHAR(by_clock);
    ac_time[1] = LOW_CHAR(by_clock);
    ac_time[2] = ':';

    by_clock = Clock_GetVal(CLOCK_MIN);
    ac_time[3] = HI_CHAR(by_clock);
    ac_time[4] = LOW_CHAR(by_clock);
    ac_time[5] = ':';

    by_clock = Clock_GetVal(CLOCK_SEC);
    ac_time[6] = HI_CHAR(by_clock);
    ac_time[7] = LOW_CHAR(by_clock);
    ac_time[8] = 0x00;

    LCD_CursorPos(0x00);
    Thread_Delay(LCD_DELAY);
    LCD_WriteText(ac_time);
    Thread_Delay(LCD_DELAY);

    by_clock = Clock_GetVal(CLOCK_DAY);
    by_clock++;
    ac_time[0] = HI_CHAR(by_clock);
    ac_time[1] = LOW_CHAR(by_clock);
    ac_time[2] = '-';

    by_clock = Clock_GetVal(CLOCK_MON);
    by_clock++;
    ac_time[3] = HI_CHAR(by_clock);
    ac_time[4] = LOW_CHAR(by_clock);
    ac_time[5] = '-';

    by_clock = Clock_GetVal(CLOCK_YEAR);
    ac_time[6] = HI_CHAR(by_clock);
    ac_time[7] = LOW_CHAR(by_clock);
    ac_time[8] = ' ';

    LCD_CursorPos(0x40);
    Thread_Delay(LCD_DELAY);
    LCD_WriteText(ac_time);
    Thread_Delay(LCD_DELAY);

    return;
    }

//------------------------------------------------

 void ThreadDisplay(void)
    {

    DisplayClear();

    while(1)
        {
        switch(gi_display_mode)
            {
            case DISPLAY_STOP:
                break;

            case DISPLAY_CLOCK:
                DisplaySystemTime();
                break;

            case DISPLAY_VALUE:
                DisplayValue();
                break;

            case DISPLAY_LINE_1:
                DisplayLine1();
                break;

            case DISPLAY_LINE_2:
                DisplayLine2();
                break;

            case DISPLAY_MSG:
                DisplayMsg();
                break;

            case DISPLAY_INSERT:
                DisplayInsert();
                break;

            case DISPLAY_MOVE_TEXT:
                DisplayMoveText();
                break;

            case DISPLAY_UNDEF_KEY:
                gac_line_1 = "Key not!";
                gac_line_2 = "Defined!";
                gw_thread_delay = 200;
                DisplayMsg();
                break;

        case DISPLAY_OUT_OF_RANGE:
                gac_line_1 = " Out of!";
                gac_line_2 = " Range !";
                gw_thread_delay = 200;
                DisplayMsg();
                break;

            default:
                DisplaySystemTime();
                break;
            }

        if(gby_display_run)
            Port_WriteBit(PORT_LED2, 1);
        else
            Port_WriteBit(PORT_LED2, 0);
        }

    return;
    }

//------------------------------------------------