/*** SetParameter.cc ***/
//------------------------------------------------
// Author: Torsten Knorr
// E-mail: create-soft@freenet.de
// Copyright (c) 2015-2017, Torsten Knorr
//------------------------------------------------

 #include "Weatherstation.h"

//------------------------------------------------

 void SetOutputPower1(void)
    {
    byte by_attempts;

    InsertCmdAreas();

    gby_wait_display = true;
    SET_DISPLAY_MOVE(
        "Set Output Power 1 (0dBm ... -21dBm)",
        "0 = max. ... 7 = min."
        )
    Thread_Delay(600);

    SET_DISPLAY_INSERT("OutPow 1", " 0 - 7  ")
    gby_cmd    = (gby_cmd | RFM12BP_CONF_OUT_1);
    gby_para_1 = (GetDecValue(1) & 0x0007);
    gby_para_2 = 0x00;

    by_attempts = 5;
    while(by_attempts--)
        {
        if(DataRequest())
            {
            SET_DISPLAY_MSG("Send OP1", "-- OK --", 200)
            break;
            }

        SET_DISPLAY_MSG("Send OP1", " ERROR! ", 200)
        Thread_Delay(200);
        }

    gbo_display_change = true;

    return;
    }

//------------------------------------------------

 void SetOutputPower2(void)
    {
    byte by_attempts;

    InsertCmdAreas();

    gby_wait_display = true;
    SET_DISPLAY_MOVE(
        "Set Output Power 2 (0dBm ... -21dBm)",
        "0 = max. ... 7 = min."
        )
    Thread_Delay(600);

    SET_DISPLAY_INSERT("OutPow 2", " 0 - 7  ")
    gby_cmd    = (gby_cmd | RFM12BP_CONF_OUT_2);
    gby_para_1 = (GetDecValue(1) & 0x0007);
    gby_para_2 = 0x00;

    by_attempts = 5;
    while(by_attempts--)
        {
        if(DataRequest())
            {
            SET_DISPLAY_MSG("Send OP2", "-- OK --", 200)
            break;
            }

        SET_DISPLAY_MSG("Send OP2", " ERROR! ", 200)
        Thread_Delay(200);
        }

    gbo_display_change = true;

    return;
    }

//------------------------------------------------

 void SetDeviation1(void)
    {
    byte by_attempts;

    InsertCmdAreas();

    gby_wait_display = true;
    SET_DISPLAY_MOVE(
        "Set Frequency Deviation (15kHz - 240kHz)",
        "0 = 15kHz ... 15 = 240kHz"
        )
    Thread_Delay(600);

    SET_DISPLAY_INSERT("FrqHup 1", "00 - 15 ")
        gby_cmd    = (gby_cmd | RFM12BP_CONF_DEV_1);
        gby_para_1 = ((GetDecValue(2) & 0x0F) << 4);
        gby_para_2 = 0x00;

    by_attempts = 5;
    while(by_attempts--)
        {
        if(DataRequest())
            {
            SET_DISPLAY_MSG("Send FH1", "-- OK --", 200)
            break;
            }

        SET_DISPLAY_MSG("Send FH1", " ERROR! ", 200)
        Thread_Delay(200);
        }

    gbo_display_change = true;

    return;
    }

//------------------------------------------------

 void SetDeviation2(void)
    {
    byte by_attempts;

    InsertCmdAreas();

    gby_wait_display = true;
    SET_DISPLAY_MOVE(
        "Set Frequency Deviation (15kHz - 240kHz)",
        "0 = 15kHz ... 15 = 240kHz"
        )
    Thread_Delay(600);

    SET_DISPLAY_INSERT("FrqHup 2", "00 - 15 ")
        gby_cmd    = (gby_cmd | RFM12BP_CONF_DEV_2);
        gby_para_1 = ((GetDecValue(2) & 0x0F) << 4);
        gby_para_2 = 0x00;

    by_attempts = 5;
    while(by_attempts--)
        {
        if(DataRequest())
            {
            SET_DISPLAY_MSG("Send FH2", "-- OK --", 200)
            break;
            }

        SET_DISPLAY_MSG("Send FH2", " ERROR! ", 200)
        Thread_Delay(200);
        }

    gbo_display_change = true;

    return;
    }

//------------------------------------------------

 void SendParaRequest(void)
    {
    byte by_attempts;

    by_attempts = 5;
    while(by_attempts--)
        {
        if(DataRequest())
            {
            SET_DISPLAY_MSG("Para. RX", "-- OK --", 50)
            break;
            }
        else
            {
            SET_DISPLAY_MSG("Para. RX", " ERROR! ", 200)
            }

        if(0 == by_attempts)
            {
            gbo_display_change = true;
            return;
            }
        }

    gby_cmd    = (RFM12BP_REMOTE_CMD | RFM12BP_CONFIRM);
    gby_para_1 = 0x00;
    gby_para_2 = 0x00;
    by_attempts = 3;
    while(by_attempts--)
        {
        if(DataRequest())
            {
            SET_DISPLAY_MSG("Confirm ", "-- OK --", 50)
            break;
            }
         else
            {
            SET_DISPLAY_MSG("Confirm ", " ERROR! ", 50)
            }
        }

    gbo_display_change = true;

    return;
    }

//------------------------------------------------

 void SetBasebandWidth(void)
    {

    InsertCmdAreas();

    gby_wait_display = true;
    SET_DISPLAY_MOVE(
        "Set Baseband Bandwidth (67kHz - 400kHz)",
        "1 = 400kHz ... 6 = 67kHz"
        )
    Thread_Delay(600);

    SET_DISPLAY_INSERT("BB-Width", " 1 - 6  ")
    gby_cmd    = (gby_cmd | RFM12BP_CONF_BBW);
    gby_para_1 = ((GetDecValue(1) & 0x07) << 5);
    gby_para_2 = 0x00;

    SendParaRequest();

    return;
    }

//------------------------------------------------

 void SetLNAGain(void)
    {

    InsertCmdAreas();

    gby_wait_display = true;
    SET_DISPLAY_MOVE(
        "Set LNA-Gain (0dBm - -20dBm)",
        "0 = 0dBm ... 3 = -20dBm"
        )
    Thread_Delay(600);

    SET_DISPLAY_INSERT("LNA-Gain", " 0 - 3  ")
    gby_cmd    = (gby_cmd | RFM12BP_CONF_LNAG);
    gby_para_1 = ((GetDecValue(1) & 0b00000011) << 3);
    gby_para_2 = 0x00;

    SendParaRequest();

    return;
    }

//------------------------------------------------

 void SetDRSSI(void)
    {

    InsertCmdAreas();

    gby_wait_display = true;
    SET_DISPLAY_MOVE(
        "Set DRSSI-threshold (-103dBm - -73dBm)",
        "0 = -103dBm ... 5 = -73dBm"
        )
    Thread_Delay(600);

    SET_DISPLAY_INSERT("DRSSI Th", " 0 - 5  ")
            gby_cmd    = (gby_cmd | RFM12BP_CONF_DRSSI);
            gby_para_1 = (GetDecValue(1) & 0x07);
            gby_para_2 = 0x00;

    SendParaRequest();

    return;
    }

//------------------------------------------------

 void SetDQD(void)
    {

    InsertCmdAreas();

    gby_wait_display = true;
    SET_DISPLAY_MOVE(
        "Set Data Quality Detector (1 - 7)",
        "0 = min. ... 7 = max."
        )
    Thread_Delay(600);

    SET_DISPLAY_INSERT("DQDedect", " 0 - 7  ")
    gby_cmd    = (gby_cmd | RFM12BP_CONF_DQD);
    gby_para_1 = (GetDecValue(1) & 0x07);
    gby_para_2 = 0x00;

    SendParaRequest();

    return;
    }

//------------------------------------------------